unit TimeClientClassUnit;

interface

uses
  Windows, Classes, ActiveX, TimeAutomationServer_TLB;

type
  TOleNotifyEvent = procedure(Sender: IUnknown) of object;

  //This is the class that will receive the event notifications
  //from the server and will then fire an event handler in response
  //Note that it is necessary to manually redefine
  //IUnknown.QueryInterface and IDispatch.Invoke
  TTimeEventSink = class(TInterfacedObject, IUnknown, IDispatch)
  private
    //This is the server object, kept here just in case...
    //(and to pass as an event handler parameter, for courtesy purposes)
    FServer: IUnknown;
    //The pointer to the event handler
    FOnRing: TOleNotifyEvent;
  protected
    { IUnknown }
    function QueryInterface(const IID: TGUID; out Obj): HRESULT; stdcall;
    { IDispatch }
    function GetTypeInfoCount(out Count: Integer): HRESULT; stdcall;
    function GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HRESULT; stdcall;
    function GetIDsOfNames(const IID: TGUID; Names: Pointer;
      NameCount, LocaleID: Integer; DispIDs: Pointer): HRESULT; stdcall;
    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
      Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HRESULT; stdcall;
  public
    //The constructor stores the server interface reference
    constructor Create(Server: IUnknown);
    //This event is a reflection of the event in the server
    property OnRing: TOleNotifyEvent read FOnRing write FOnRing;
  end;

implementation

{ TTimeEventSink }

constructor TTimeEventSink.Create(Server: IUnknown);
begin
  inherited Create;
  FServer := Server;
end;

//These IDispatch methods are not of interest here, so they are stubbed out
function TTimeEventSink.GetIDsOfNames(const IID: TGUID; Names: Pointer;
  NameCount, LocaleID: Integer; DispIDs: Pointer): HRESULT;
begin
  Result := E_NOTIMPL;
end;

function TTimeEventSink.GetTypeInfo(Index, LocaleID: Integer;
  out TypeInfo): HRESULT;
begin
  Result := E_NOTIMPL;
end;

function TTimeEventSink.GetTypeInfoCount(out Count: Integer): HRESULT;
begin
  Count  := 0;
  Result := S_OK;
end;

//The server will call this to trigger the event(s).
//The event interface is a dispatch interface, and so we
//can trap the event call in IDispatch.Invoke (i.e. here)
function TTimeEventSink.Invoke(DispID: Integer; const IID: TGUID;
  LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo,
  ArgErr: Pointer): HRESULT;
begin
  //Fire the different event handlers when
  //the different event methods are invoked
  case DispID of
    1 : if Assigned(FOnRing) then
          FOnRing(FServer);
  end;
  Result := S_OK;
end;

function TTimeEventSink.QueryInterface(const IID: TGUID; out Obj): HRESULT;
begin
  //We need to return the event interface when it is asked for
  Result := E_NOINTERFACE;
  //Try and get any interface requested
  if GetInterface(IID, Obj) then
    Result := S_OK;
  //When the server asks for IAlarmEvents, we pass back our IDispatch interface
  if IsEqualGUID(IID, IAlarmEvents) and GetInterface(IDispatch, Obj) then
    Result := S_OK;
end;

end.
