unit TimeClientFormUnit;

{ Consider a button and its OnClick event. We'll call the button a
server for now. When it fires the OnClick event, the event handler
lives elsewhere, typically in a form if the event handler was set
up at design-time. The form can be considered a client. The IDE can
work out what event handlers there are, and how they look by
examining the button's RTTI.

In the world of COM, an Automation server may well have events it
fires. The event handlers will live in a client application. To
find out what event handlers there are, and how they look, we can
examine the server's type library. The events are defined in an
interface on the server (referred to as a connectable interface).
The client picks up the fact that events are fired by implementing
an interface as well. This interface is called a sink.

The sink class must implement IUnknown and IDispatch, specifically
supplying a custom QueryInterface method, to react to queries about
the events interface and a custom Invoke method which picks up the
events firing and can then trigger the event handlers. To connect
the sink to the connectable class, use ComObj.InterfaceConnect. }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, TimeClientClassUnit, TimeAutomationServer_TLB, ExtCtrls,
  Buttons;

type
  TForm2 = class(TForm)
    Label1: TLabel;
    Timer1: TTimer;
    Label2: TLabel;
    SpeedButton1: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
  public
    ServerApp: ITimeServer;
    ClientEventSink: TTimeEventSink;
    ConnectionCookie: Integer;
    procedure AlarmRing(Sender: IUnknown);
  end;

var
  Form2: TForm2;

implementation

uses
  ComObj;

{$R *.DFM}

procedure TForm2.FormCreate(Sender: TObject);
begin
  //Connect to the server
  ServerApp := CoTimeServer.Create;
  //Make an event sink object
  ClientEventSink := TTimeEventSink.Create(ServerApp);
  //Set up the event handler
  ClientEventSink.OnRing := AlarmRing;
  //Hook the event sink up to the server (TimeServer)
  InterfaceConnect(ServerApp, IAlarmEvents, ClientEventSink, ConnectionCookie);
end;

procedure TForm2.FormDestroy(Sender: TObject);
begin
  //Unhook the event sink from the server (TimeServer)
  InterfaceDisconnect(ServerApp, IAlarmEvents, ConnectionCookie);
end;

procedure TForm2.Timer1Timer(Sender: TObject);
begin
  try
    //This just accesses a couple of server properties
    Label1.Caption := TimeToStr(ServerApp.CurrentTime);
    Label2.Caption := DateToStr(ServerApp.CurrentDate);
  except
    Label1.Caption := 'Cannot access time';
    Label2.Caption := 'Cannot access date';
    (Sender as TTimer).Enabled := False
  end
end;

procedure TForm2.SpeedButton1Click(Sender: TObject);
begin
  //Call the server routine and ask it to fire an event in 5 seconds
  (ServerApp as IAlarm).SetAlarm(ServerApp.CurrentTime + 5/SecsPerDay);
end;

procedure TForm2.AlarmRing(Sender: IUnknown);
begin
  //This is the event handler
  ShowMessage('Rrriinnggggg!!')
end;

end.
