unit TimeServerClassUnit;

interface

uses
  ComObj, ActiveX, AxCtrls, TimeAutomationServer_TLB, ExtCtrls;

type
  //The auto-generated code here saves us the trouble of
  //working out how to connect to the client's event sink
  TTimeServer = class(TAutoObject, IConnectionPointContainer, ITimeServer, IAlarm)
  private
    { Private declarations }
    FConnectionPoints: TConnectionPoints;
    //Pointer to event sink
    FEvents: IAlarmEvents;
    FTimer: TTimer;
    FAlarmTime: TDateTime;
    procedure FOnTimer(Sender: TObject);
  protected
    { Protected declarations }
    property ConnectionPoints: TConnectionPoints read FConnectionPoints
      implements IConnectionPointContainer;
    procedure EventSinkChanged(const EventSink: IUnknown); override;
    function Get_CurrentDate: TDateTime; safecall;
    function Get_CurrentTime: TDateTime; safecall;
  public
    procedure Initialize; override;
    destructor Destroy; override;
    procedure SetAlarm(AlarmTime: TDateTime); safecall;
    procedure CancelAlarm; safecall;
  end;

implementation

uses ComServ, SysUtils;

procedure TTimeServer.Initialize;
begin
  inherited Initialize;
  FConnectionPoints := TConnectionPoints.Create(Self);
  if AutoFactory.EventTypeInfo <> nil then
    FConnectionPoints.CreateConnectionPoint(AutoFactory.EventIID,
      ckSingle, EventConnect);
end;

destructor TTimeServer.Destroy;
begin
  FTimer.Free;
  FTimer := nil;
  inherited
end;

procedure TTimeServer.EventSinkChanged(const EventSink: IUnknown);
begin
  FEvents := EventSink as IAlarmEvents;
end;

function TTimeServer.Get_CurrentDate: TDateTime;
begin
  Result := Date
end;

function TTimeServer.Get_CurrentTime: TDateTime;
begin
  Result := Time
end;

procedure TTimeServer.SetAlarm(AlarmTime: TDateTime);
begin
  FAlarmTime := AlarmTime;
  if FTimer = nil then
  begin
    FTimer := TTimer.Create(nil);
    FTimer.Interval := 100;
    FTimer.OnTimer := FOnTimer;
  end;
  FTimer.Enabled := True
end;

procedure TTimeServer.CancelAlarm;
begin
  if Assigned(FTimer) then
    FTimer.Enabled := False
end;

procedure TTimeServer.FOnTimer(Sender: TObject);
begin
  if Time >= FAlarmTime then
  begin
    CancelAlarm;
    if FEvents <> nil then
      FEvents.Ring
  end
end;

initialization
  TAutoObjectFactory.Create(ComServer, TTimeServer, Class_TimeServer,
    ciMultiInstance, tmApartment);
end.
