unit HeapMonitorUnit;

interface


function MakeCaption(const Caption: String): String;

var
  HeapBlocks: Cardinal = 0;
  HeapBytes: Cardinal = 0;
  HeapError: Cardinal = 0;
  HeapMonitor: Boolean = False;
  HeapVerify: Boolean = False;

implementation

uses
  SysUtils, Windows;

var
  RTLMemMgr: TMemoryManager;

const
  StartingOffset: Cardinal = 0;

function GetHeapBytes: Cardinal;
var
  HS: THeapStatus;
begin
  HS := GetHeapStatus;
  HeapError := HS.HeapErrorCode;
  Result := HS.TotalAllocated;
  HeapBytes := HS.TotalAllocated - StartingOffset;
end;

function MakeCaption(const Caption: String): String;
const
  Total = '%s (Blocks=%.0n, Bytes=%.0n)';
  Error = '%s (**Invalid Heap** code %d)';
begin
  if HeapError = 0 then
  begin
    if HeapMonitor then
      Result := Format(Total, [Caption, HeapBlocks * 1.0, HeapBytes * 1.0])
  end
  else
    if HeapVerify then
      Result := Format(Error, [Caption, HeapError]);
end;

function NewGetMem(Size: Integer): Pointer;
begin
  Result := RTLMemMgr.GetMem(Size);
  Inc(HeapBlocks);
  GetHeapBytes
end;

function NewFreeMem(P: Pointer): Integer;
begin
  Result := RTLMemMgr.FreeMem(P);
  Dec(HeapBlocks);
  GetHeapBytes
end;

function NewReallocMem(P: Pointer; Size: Integer): Pointer;
begin
  Result := RTLMemMgr.ReallocMem(P, Size);
  GetHeapBytes
end;

const
  SwitchChars = ['-', '/'];
  NewMemMgr: TMemoryManager = (GetMem: NewGetMem;
    FreeMem: NewFreeMem; ReallocMem: NewReallocMem);

var
  S: String;

initialization
  StartingOffset := GetHeapBytes;
  if FindCmdLineSwitch('HM', SwitchChars, True) then
  begin
    HeapMonitor := True;
    GetMemoryManager(RTLMemMgr);
    SetMemoryManager(NewMemMgr);
  end;
  if FindCmdLineSwitch('HV', SwitchChars, True) then
    HeapVerify := True;
finalization
  if (HeapBlocks <> 0) and (HeapBytes <> 0) then
  begin
    S := Format('%.0n bytes were not freed (%.0n allocations)',
      [HeapBytes*1.0, HeapBlocks*1.0]);
    MessageBox(0, PChar(S), 'Heap Integrity Issue', MB_OK)
  end;
  StartingOffset := 0;
end.
