unit AggregationServerClassesU;

{ Implements aggregation where the aggregated object is
  based on TAggregatedObject. The aggregated may only be
  used from within the controlling COM object, and cannot
  be instantiated directly by a COM client }

interface

uses
  ComObj, AggregationServerInterfaceU;

type
  TAggreg = class
  public
    constructor Create;
    destructor Destroy; override;
    procedure AnAggrMethod; safecall;
  end;

  TControlling = class(TComObject, IControlling, IAggr)
  private
    FAggreg: TAggreg;
    function GetAggreg: TAggreg;
  public
    procedure Initialize; override;
    destructor Destroy; override;
    procedure AControllingMethod; safecall;
    property Aggreg: TAggreg read GetAggreg implements IAggr;
  end;

implementation

uses
  ComServ, Dialogs, ActiveX, Windows;

{ TAggreg }

constructor TAggreg.Create;
begin
  inherited;
  ShowMessage('Creating a ' + ClassName);
end;

destructor TAggreg.Destroy;
begin
  ShowMessage('Destroying a ' + ClassName);
  inherited
end;

procedure TAggreg.AnAggrMethod;
begin
  ShowMessage('An Aggregated Method: ' + ClassName);
end;

{ TControlling }

procedure TControlling.Initialize;
begin
  inherited;
  ShowMessage('Creating a ' + ClassName);
end;

destructor TControlling.Destroy;
begin
  FAggreg.Free;
  ShowMessage('Destroying a ' + ClassName);
  inherited
end;

function TControlling.GetAggreg: TAggreg;
begin
  if FAggreg = nil then
    FAggreg := TAggreg.Create;
  Result := FAggreg
end;

procedure TControlling.AControllingMethod;
begin
  ShowMessage('A Controlling Method: ' + ClassName);
end;

initialization
  TComObjectFactory.Create(ComServer, TControlling, Class_Controlling, 'Controller',
  'This is a controlling object', ciMultiInstance);
end.
